'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var CoreApp = require('fusion-core');
var CoreApp__default = _interopDefault(CoreApp);
var fusionReactAsync = require('fusion-react-async');
var ReactDOM = _interopDefault(require('react-dom'));
var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));

/* eslint-env node */

/* eslint-env browser */
var clientRender = (el => {
  const domElement = document.getElementById('root');
  ReactDOM.hydrate ? ReactDOM.hydrate(el, domElement) : ReactDOM.render(el, domElement);
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var Provider = {
  create: (name, BaseComponent = React.Component) => {
    class Provider extends BaseComponent {
      getChildContext() {
        return { [name]: this.props.service };
      }
      render() {
        return React.Children.only(this.props.children);
      }
    }
    Provider.childContextTypes = _extends({}, Provider.childContextTypes || {}, {
      [name]: PropTypes.object.isRequired
    });
    Provider.displayName = name.replace(/^./, c => c.toUpperCase()) + 'Provider';

    return Provider;
  }
};

var plugin = {
  create: (name, plugin, BaseComponent) => (...args) => {
    const p = plugin(...args);
    const middleware = p.middleware;
    const ProviderComponent = Provider.create(name, BaseComponent);
    p.middleware = function (ctx, next) {
      if (ctx.element) {
        ctx.element = React.createElement(ProviderComponent, { service: this.of(ctx) }, ctx.element);
      }
      return middleware.call(p, ctx, next);
    };
    return p;
  }
};

var _extends$1 = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var hoc = {
  create: (name, mapServiceToProps) => {
    if (!mapServiceToProps) {
      mapServiceToProps = service => ({ [name]: service });
    }
    return Component => {
      class HOC extends React.Component {
        constructor(props, ctx) {
          super(props, ctx);
          this.service = ctx[name];
        }
        render() {
          const props = _extends$1({}, this.props, mapServiceToProps(this.service));
          return React.createElement(Component, props);
        }
      }
      const displayName = Component.displayName || Component.name;
      HOC.displayName = 'With' + name.replace(/^./, c => c.toUpperCase()) + '(' + displayName + ')';
      HOC.contextTypes = {
        [name]: PropTypes.object.isRequired
      };
      return HOC;
    };
  }
};

/* eslint-env browser */
class App extends CoreApp__default {
  constructor(root) {
    super(root, el => {
      return fusionReactAsync.prepare(el).then(() => {
        return clientRender(el);
      });
    });
    this.plugins.push(CoreApp.compose([fusionReactAsync.PreparePlugin(), this.plugins.pop()]));
  }
}

exports['default'] = App;
exports.ProviderPlugin = plugin;
exports.ProvidedHOC = hoc;
exports.Provider = Provider;
//# sourceMappingURL=browser.cjs.es2015.js.map
