import CoreApp, { compose } from 'fusion-core';
import { PreparePlugin, prepare } from 'fusion-react-async';
import { renderToString } from 'react-dom/server';
import React from 'react';
import PropTypes from 'prop-types';

/* eslint-env node */
var serverRender = (el => `<div id='root'>${renderToString(el)}</div>`);

/* eslint-env browser */

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var Provider = {
  create: (name, BaseComponent = React.Component) => {
    class Provider extends BaseComponent {
      getChildContext() {
        return { [name]: this.props.service };
      }
      render() {
        return React.Children.only(this.props.children);
      }
    }
    Provider.childContextTypes = _extends({}, Provider.childContextTypes || {}, {
      [name]: PropTypes.object.isRequired
    });
    Provider.displayName = name.replace(/^./, c => c.toUpperCase()) + 'Provider';

    return Provider;
  }
};

var plugin = {
  create: (name, plugin, BaseComponent) => (...args) => {
    const p = plugin(...args);
    const middleware = p.middleware;
    const ProviderComponent = Provider.create(name, BaseComponent);
    p.middleware = function (ctx, next) {
      if (ctx.element) {
        ctx.element = React.createElement(ProviderComponent, { service: this.of(ctx) }, ctx.element);
      }
      return middleware.call(p, ctx, next);
    };
    return p;
  }
};

var _extends$1 = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var hoc = {
  create: (name, mapServiceToProps) => {
    if (!mapServiceToProps) {
      mapServiceToProps = service => ({ [name]: service });
    }
    return Component => {
      class HOC extends React.Component {
        constructor(props, ctx) {
          super(props, ctx);
          this.service = ctx[name];
        }
        render() {
          const props = _extends$1({}, this.props, mapServiceToProps(this.service));
          return React.createElement(Component, props);
        }
      }
      const displayName = Component.displayName || Component.name;
      HOC.displayName = 'With' + name.replace(/^./, c => c.toUpperCase()) + '(' + displayName + ')';
      HOC.contextTypes = {
        [name]: PropTypes.object.isRequired
      };
      return HOC;
    };
  }
};

/* eslint-env browser */
class App extends CoreApp {
  constructor(root) {
    super(root, el => {
      return prepare(el).then(() => {
        return serverRender(el);
      });
    });
    this.plugins.push(compose([PreparePlugin(), this.plugins.pop()]));
  }
}

export { plugin as ProviderPlugin, hoc as ProvidedHOC, Provider };export default App;
//# sourceMappingURL=node.es.es2015.js.map
